﻿using System;
using Xunit;
using Moq;
using SubSpec;

public class CompositeFixtureFacts
{
    [Fact]
    public void Dispose_Disposes_All_Registered_Fixtures()
    {
        var disposableA = new Mock<IDisposable>();
        var disposableB = new Mock<IDisposable>();

        var sut = new CompositeFixture() { disposableA.Object, disposableB.Object };

        sut.Dispose();

        disposableA.Verify( x => x.Dispose() );
        disposableB.Verify( x => x.Dispose() );
    }

    [Fact]
    public void Dispose_Continues_After_Exception()
    {
        var disposableA = new Mock<IDisposable>();
        var disposableB = new Mock<IDisposable>();

        disposableA.Setup( x => x.Dispose() ).Throws( new Exception() );

        var sut = new CompositeFixture() { disposableA.Object, disposableB.Object };

        var _ = Record.Exception( () => sut.Dispose() );

        disposableA.Verify( x => x.Dispose() );
        disposableB.Verify( x => x.Dispose() );
    }

    [Fact]
    public void Dispose_Records_All_Exceptions()
    {
        var disposableA = new Mock<IDisposable>();
        var disposableB = new Mock<IDisposable>();

        disposableA.Setup( x => x.Dispose() ).Throws( new Exception() );
        disposableB.Setup( x => x.Dispose() ).Throws( new InvalidOperationException() );

        var sut = new CompositeFixture() { disposableA.Object, disposableB.Object };

        var exception = Record.Exception( () => sut.Dispose() );

        var agg = Assert.IsType<CompositeFixture.AggregateException>( exception );
        Assert.Equal( 2, agg.InnerExceptions.Count );
        Assert.IsType<Exception>( agg.InnerExceptions[1] );
        Assert.IsType<InvalidOperationException>( agg.InnerExceptions[0] );
    }

    [Fact]
    public void AggregateException_ToString_Is_Useful()
    {
        var exceptions = new Exception[] { new Exception( "ExceptionA" ), new Exception( "ExceptionB" ) };

        var sut = new CompositeFixture.AggregateException( exceptions );

        string result = sut.ToString();
        Assert.NotEmpty( result );
        Assert.Contains( "ExceptionA", result );
        Assert.Contains( "ExceptionB", result );
    }
}
